/*
 * Name:    example3.c
 *
 * Purpose: This program shows how to create a new dark current image frame
 *          for an area detector.
 *
 * Author:  William Lavender
 *
 *--------------------------------------------------------------------------
 *
 * Copyright 2006 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include <stdio.h>
#include <stdlib.h>

#include "mx_util.h"
#include "mx_record.h"
#include "mx_image.h"
#include "mx_area_detector.h"

int
main( int argc, char *argv[] )
{
        MX_RECORD *mx_database, *ad_record;
        MX_IMAGE_FRAME *image_frame;
        char mx_database_name[] = "./mx_client.dat";
        char area_detector_name[] = "ad";
        double exposure_time;
        long num_exposures;
        char *savefile_name;
        mx_bool_type busy;
        mx_status_type mx_status;

        if ( argc != 4 ) {
                fprintf(stderr,
                "Usage: %s 'exposure time' 'num exposures' 'savefile name'\n",
                                argv[0]);
                exit(1);
        }

        image_frame = NULL;

        exposure_time = atof( argv[1] );
        num_exposures = atoi( argv[2] );
        savefile_name = argv[3];

        /* Initialize the MX database. */

        mx_status = mx_setup_database( &mx_database, mx_database_name );

        if ( mx_status.code != MXE_SUCCESS ) {
                fprintf( stderr,
                        "Did not successfully initialize MX database '%s'.\n",
                        mx_database_name );
                exit(1);
        }

        /* Find the area detector record in the database. */

        ad_record = mx_get_record( mx_database, area_detector_name );

        if ( ad_record == NULL ) {
                fprintf( stderr,
                "Did not find area detector record '%s' in MX database '%s'.\n",
                        area_detector_name, mx_database_name );
                exit(1);
        }

        /* Tell the area detector to start acquiring images with which
         * to make a dark current correction frame.
         *
         * There should be no photons hitting the detector while this
         * measurement is in progress.
         */

        if ( num_exposures == 1 ) {
                fprintf( stderr,
                "Starting the dark current measurement using "
                "1 exposure of %g seconds.\n",
                        exposure_time );
        } else {
                fprintf( stderr,
                "Starting the dark current measurement using "
                "%ld exposures of %g seconds each.\n",
                        num_exposures, exposure_time );
        }

        mx_status = mx_area_detector_measure_dark_current_frame( ad_record,
                                                exposure_time, num_exposures );

        if ( mx_status.code != MXE_SUCCESS ) {
                fprintf( stderr,
                "Unable to start the dark current measurement.\n" );
                exit(1);
        }

        /* Wait for the dark current measurement to complete. */

        while (1) {
                /* See if the measurement is still in progress. */

                mx_status = mx_area_detector_is_busy( ad_record, &busy );

                if ( mx_status.code != MXE_SUCCESS ) {
                        fprintf( stderr,
                    "An attempt to check the status of the detector failed.\n");
                        exit(1);
                }

                if ( busy == FALSE ) {
                        /* The measurement is complete, so we can now
                         * break out of the while() loop.
                         */

                        break;
                }

                /* Sleep for a millisecond so that we do not use up
                 * all of the CPU time on the computer.
                 */

                mx_msleep(1);
        }

        fprintf( stderr, "The dark current measurement is complete.\n" );

        /* The new dark current frame will already be in the correct
         * image buffer on the detector computer for the purpose of
         * automatic image correction.  However, we will also save a
         * copy of the new dark current frame to disk.  Please note
         * that the filename for the save file refers to the disk
         * on the detector computer.
         */

        mx_status = mx_area_detector_save_frame( ad_record,
                                                MXFT_AD_DARK_CURRENT_FRAME,
                                                savefile_name );

        if ( mx_status.code != MXE_SUCCESS ) {
                fprintf( stderr,
                "The attempt to save the new dark current frame\n"
                "to the file '%s' on the detector computer failed.\n",
                        savefile_name );
                exit(1);
        }

        fprintf( stderr,
        "The dark current frame was successfully saved to the file '%s' "
        "on the detector computer.\n",
                savefile_name );

        fprintf( stderr, "Program complete.\n" );

        exit(0);
}

